@@ -14,7 +14,7 @@ def preprocess(job_directory, setup_actions, action_executor, object_store=None)
         if getattr(action, "inject_object_store", False):
             action.object_store = object_store
         path = job_directory.calculate_path(name, input_type)
-        description = "Staging %s '%s' via %s to %s" % (input_type, name, action, path)
+        description = "Staging {} '{}' via {} to {}".format(input_type, name, action, path)
         log.debug(description)
         action_executor.execute(lambda: action.write_to_path(path), "action[%s]" % description)
 
